var debugTextArea = null;
var websocket = null;

/*
Debug function that will dump messages into a <textarea id="debugTextArea">
element if there is one on the page (see code just after $(document).ready)
*/
function debug(message) {
	 if (debugTextArea) {
		 debugTextArea.value += message + "\n";
		 debugTextArea.scrollTop = debugTextArea.scrollHeight;
	 }
}

/*
Global function to send a message to the FleckUV app, for forwarding to
Uniview. Commands that can be sent are the commands found in the Uniview runtime syntax.

If there is no websocket or if the connection isn't ready yet (could happen during
page load, if there are sendMessage calls in the page initialization), the function
sleeps for 500 ms and then tries one more time, but then discards the message. Room for
improvement, for sure.
*/
function sendMessage( strToSend, retry ) {
	if (websocket == null || websocket.readyState != 1) {
		setTimeout(function(){
			try
			{
				if (!retry)
					sendMessage(strToSend, true);
			}
			catch(err)
			{
				alert("error:" + websocket.readyState);
			}
		},500);
	}
	else {
		if ( websocket != null ) {
			websocket.send( strToSend );
			debug(strToSend);
		}
		else {
			initWebsocket();
			alert("websocket error");
		}
	}
}

/*
Uses the new HTML5 websocket API to establish a connection to FleckUV which is
a tray app based on the Fleck websocket server, in C#.
*/
function initWebsocket() {
	try {
		if (typeof MozWebSocket == 'function')
			WebSocket = MozWebSocket;

		var ip = "<?php echo $_SERVER['SERVER_ADDR']; ?>";
		var wsUri = "ws://" + ip + ":8090"
/*OLD:
		var ip = "192.168.1.146:8090";
		var wsUri = "ws://" + ip;
END OLD*/
		websocket = new WebSocket(wsUri);

		websocket.onopen = function (evt) {
			debug("CONNECTED");
		};

		websocket.onclose = function (evt) {
			debug("DISCONNECTED");
		};

		/*This is a stub, at the moment Uniview does not return any
		values and the FleckUV trayApp doesn't send anything back to
		clientside javascript, so nothing ever gets here*/
		websocket.onmessage = function (evt) {
			handlePropertyChange(evt.data);
		};

		websocket.onerror = function (evt) {
			debug('ERROR: ' + evt.data);
		};
	} catch (exception) {
		debug('ERROR: ' + exception);
	}
}

/*
Unlikely to ever be called I guess, but here is how we gracefully
disconnect from the tray app.
*/
function stopWebsocket() {
	if (websocket)
		websocket.close();
}

/*
This is a stub, there are no return messages from Uniview
*/
function handlePropertyChange( json ) {
}

/*
Variables used by the Swipe functionality.
*/
var slider;
var selectors;

/*
Function used when setting up slider logic. a Swipe 1.0
example, by Brad Birdsall, Prime. Copyright 2011, Licensed GPL & MIT
*/
function setTab(elem)
{
	for (var i =0; i < selectors.length; i++)
	{
		selectors[i].className = selectors[i].className.replace('uv-selected',' ');
	}
	elem.className += ' uv-selected';

	$(".uv-widget").removeClass('hidden');

	// var dattr = $(elem).attr('data-disabled-widgets');

	// if (dattr)
	// {
		// var disabled = dattr.split(";");
		// for (var i = 0; i < disabled.length; i++)
		// {
			// $("#"+disabled[i]).addClass('hidden');
		// }
	// }
}

/*
Alright, document.ready is the starting point. Here is what happens, sequentially,
when the page is loaded.
*/
$(document).ready(function() {
	/*
	See if the HTML author has allocated a debug area or not. If there
	is one, i.e. if the HTML document contains a <textarea id="debugTextArea">
	tag, it will be used for output.
	*/
	debugTextArea = document.getElementById("debugTextArea");

	/*
	Set up the slider logic. This code comes from the Swipe 1.0
	example, by Brad Birdsall, Prime. Copyright 2011, Licensed GPL & MIT
	*/
	slider = new Swipe(document.getElementById('tabs'),
	{
		callback: function(event, index, elem) {
			setTab(selectors[index]);
		}
	}),
	selectors = document.getElementById('tabSelector').children;

	for (var i=0; i < selectors.length; i++)
	{
		var elem = selectors[i];
		elem.setAttribute('data-tab', i);
		elem.onclick = function(e)
		{
			e.preventDefault();
			setTab(this);
			slider.slide(parseInt(this.getAttribute('data-tab'), 10),300);
		}
	}
	/*
	End of slider logic
	*/

	/*
	Create that websocket connection
	*/
	initWebsocket();

	/*
	Initialize touchpads, w.o. jquery
	touchright
	*/
	var touchStart = new Array();
	var touchOutput = [0.0, 0.0, 0.0, 0.0, 0.0];


	/*
	We have first logic to control the camera, i.e. emulate the normal
	Uniview mouse input, from touch elements in HTML.
	*/
	function updateCamera()
	{
		if (touchOutput[0] != 0.0 || touchOutput[1] != 0.0 || touchOutput[2] != 0.0)
			sendMessage("camera.rotate " + touchOutput[0] + " " + touchOutput[1] + " " + touchOutput[2]);
		if (touchOutput[3] != 0.0 || touchOutput[4] != 0.0)
			sendMessage("camera.translate " + touchOutput[3] + " " + touchOutput[4] + " 0.0");
		if (touchOutput[0] == 0.0 && touchOutput[1] == 0.0 && touchOutput[2] == 0.0 && touchOutput[3] == 0.0 && touchOutput[4] == 0.0)
			sendMessage("camera.noinput");
	}

	$("#touchleft").bind("touchstart", function (event) {
		var oe = event.originalEvent;
		if (oe.touches.length > 0)
		{
			var index = 0;
			if (oe.touches.length > 1)
			{
				if (oe.touches[1].pageX < oe.touches[0].pageX)
					index = 1;
			}
			touchStart[0] = parseInt(oe.touches[index].pageX);
			touchStart[1] = parseInt(oe.touches[index].pageY);
		}
	});

	$("#touchleft").bind("touchmove", function (event) {
		event.preventDefault();
		var oe = event.originalEvent;
		if (oe && oe.touches.length > 0)
		{
			var index = 0;
			if (oe.touches.length > 1)
			{
				if (oe.touches[1].pageX < oe.touches[0].pageX)
					index = 1;
			}
			touchOutput[0] = (parseInt(oe.touches[index].pageX) - touchStart[0]) / 100.0;
			touchOutput[1] = (touchStart[1] - parseInt(oe.touches[index].pageY)) / 100.0;
			touchOutput[2];
			updateCamera();
		}
	});

	$("#touchleft").bind("touchend", function (event) {
		touchOutput[0] = 0.0;
		touchOutput[1] = 0.0;
		touchOutput[2] = 0.0;
		updateCamera();
	});


	$("#touchright").bind("touchstart", function (event) {
		var oe = event.originalEvent;
		if (oe.touches.length > 0)
		{
			var index = 0;
			if (oe.touches.length > 1)
			{
				if (oe.touches[1].pageX > oe.touches[0].pageX)
					index = 1;
			}
			touchStart[2] = parseInt(oe.touches[index].pageX);
			touchStart[3] = parseInt(oe.touches[index].pageY);
		}
	});

	$("#touchright").bind("touchmove", function (event) {
		event.preventDefault();
		var oe = event.originalEvent;
		if (oe && oe.touches.length > 0)
		{
			var index = 0;
			if (oe.touches.length > 1)
			{
				if (oe.touches[1].pageX > oe.touches[0].pageX)
					index = 1;
			}
			touchOutput[3] = 0;
			touchOutput[4] = (touchStart[3] - parseInt(oe.touches[index].pageY)) / 100.0;
			updateCamera();
		}
	});

	$("#touchright").bind("touchend", function (event) {
		touchOutput[3] = 0.0;
		touchOutput[4] = 0.0;
		updateCamera();
	});
	/*
	End of camera logic.
	*/


	/*
		Message buttons:

		These are stupid buttons with no feedback or logic, just sends
		a string message
	*/
	$(".uv-message-button").each(function(index) {
		$(this).click(function() {
			var msg = $(this).attr('data-message');
			sendMessage(msg);
		});
	});
	
	/* 
		Toggle button:
		In theory this will create a button that toggles between two states, sending a different data message at each.
	*/
	
	$.fn.clicktoggle = function(a, b) {
		return this.each(function() {
			var clicked = false;
			$(this).click(function() {
				if (clicked) {
					clicked = false;
					return b.apply(this, arguments);
				}
				clicked = true;
				return a.apply(this, arguments);
			});
		});
	};
	
	function on() {
		$(this).each(function() {
				var msg = $(this).attr('data-message-on');
				sendMessage(msg);			
		});
	$(this).toggleClass('uv-toggle-on');
	}

	function off() {
		$(this).each(function() {
				var msg = $(this).attr('data-message-off');
				sendMessage(msg);		
		});
	$(this).toggleClass('uv-toggle-on');
	}

	$(".uv-toggle-off, .uv-toggle-on").clicktoggle(on,off);	
	
	/*Uniview reset button - this should reset any active toggles. It just clicks any "on" buttons
	*/
	$(".uv-reset-button").click(function() {
        $('.uv-toggle-on').click()
	});
	

	/*The delayed message accepts a json string where each object is delay in milliseconds followed by a comand
	*/
	$(".uv-delayed-message-button").each(function(index) {
		$(this).click(function() {
			var jsonstring = $(this).attr('data-message');
			if (jsonstring != null)
			{
				this.disabled = true;

				var obj = JSON.parse(jsonstring);
				for (var delay in obj)
				{
					setTimeout("sendMessage('"+obj[delay]+"')", delay);
				}
				var self = this;
				setTimeout(function() {
					self.disabled = false;
				}, delay);
			}
		});
	});
	

	/*The delayed sequence triggers a sequence of messages, each fired off at tDelta * iterator from the tStart
	time.
	*/
	$(".uv-delayed-sequence-button").each(function(index) {
		$(this).click(function() {
			var tStart = parseFloat($(this).attr('data-tstart'));
			var tDelta = parseFloat($(this).attr('data-tdelta'));
			var tSteps = parseFloat($(this).attr('data-tsteps'));
			var dStart = parseFloat($(this).attr('data-dstart'));
			var dIncrm = parseFloat($(this).attr('data-dincrm'));
			var message = $(this).attr('data-sequenced-message');

			for (var i = 0; i <tSteps; i++)
			{
				var t = tStart + tDelta*i;

				var val = dStart + dIncrm*i;
				var smsg = message.replace("%v", val);
				setTimeout("sendMessage('"+smsg+"')", t);
			}
		});
	});

	/*The bind slider button triggers a behavior (actually, a string with one or more %v replaceable floats) on the
	universal slider
	*/
	$(".uv-bind-slider-button").each(function(index)
	{
		$(this).click(function()
		{
			var s_msg = $(this).attr('data-slider-message');
			var s_mu_msg = $(this).attr('data-slider-mouseup-message');
			var s_min = $(this).attr('data-slider-min');
			var s_max = $(this).attr('data-slider-max');
			var s_step = $(this).attr('data-step');
			var s_val = $(this).attr('data-value');

			$("#uv-universal-slider").attr('min', s_min);
			$("#uv-universal-slider").attr('max', s_max);
			$("#uv-universal-slider").attr('step', s_step);
			$("#uv-universal-slider").attr('value', s_val);
			$("#uv-universal-slider").attr('data-message', s_msg);
			$("#uv-universal-slider").attr('data-mouseup-message', s_mu_msg);

			var title = $(this).attr('data-slider-title');
			if (!title || title == "")
			{
				title ="no title";
			}
			$("#uv-universal-slider-title").text(title);

			$("#uv-universal-slider").change(function()
			{
				var message = $(this).attr('data-message');
				var val = $(this).val();
				var smsg = message.replace("%v", val);
				sendMessage(smsg);
			});
			var mouseupmessage = $(this).attr('data-mouseup-message');

			if (mouseupmessage && mouseupmessage != "")
			{
				$("#uv-universal-slider").mouseup(function()
				{
					sendMessage(mouseupmessage);
					$(this).val(0.0);
				});
				$("#uv-universal-slider").bind("touchend", function(e)
				{
					sendMessage(mouseupmessage);
					$(this).val(0.0);
				});
			}
			else
			{
				$("#uv-universal-slider").unbind('mouseup');
				$("#uv-universal-slider").unbind('touchend');
			}
		});
	});

	/*
		Daily and annual motion sliders:

		Default start time: Now.
		Default end time: Now + 24hrs.

		The logic is simple, as follows:
		t0 = now, at document.ready()
		tN = t0 + annualMotion*a_full_year + dailyMotion*a_full_day + timeZoneOffset*a_full_minute

		The reason why we get UTC time and add timeZoneOffset instead
		of some other way around is we only want to get the timezone offset once and ignore
		daylight savings time. The alternative is to get daylight savings time but then also suffer
		a non-smooth and clearly visible annual motion jump at daylight savings time switch.

		JS counts months from 0-11, we add +1 at the
		very end of the logic (when sending to Uniview / printout)

		JS deals with daylight savings time and such.

		NOTE! I HAVEN'T USED THESE SLIDERS IN ANY LESSONS AS THEY SET TIME HARD AND NOT SOFT THROUGH
		THE SIMULATEDTIMEVELOCITY.
	*/

	var td = new Date();

	var t0_year = td.getUTCFullYear();
	var t0_month = td.getUTCMonth();
	var t0_date = td.getUTCDate();
	var t0_hour = td.getUTCHours();
	var t0_min = td.getUTCMinutes();
	var tz_off = -td.getTimezoneOffset();

	var annualMotion = 0;
	var dailyMotion = 0;

	$(".uv-dailymotion-slider").each(function(index) {

		$(this).change( function() {

			dailyMotion = $(this).val();
			var t = new Date(t0_year, t0_month, t0_date, t0_hour, t0_min, 0, 0);
			t.setTime(t.getTime() + annualMotion*365*24*60*60*1000 + dailyMotion*24*60*60*1000 + tz_off*60*1000);

			//Fix trailing zero on hour and minute printout
			var tmp_day_str = t.getUTCDate().toString();
			var tmp_hrs_str = t.getUTCHours().toString();
			var tmp_min_str = t.getUTCMinutes().toString();

			if (tmp_hrs_str.length == 1)
				tmp_hrs_str = "0" + tmp_hrs_str;
			if (tmp_min_str.length == 1)
				tmp_min_str = "0" + tmp_min_str;
			//End fix

			$(".uv-dailymotion-slider").val($(this).val());

			$(".uv-dailymotion-slider-printout").text("Time: " + tmp_hrs_str + ":" + tmp_min_str);
			sendMessage("system.setsimulatedtime " + t.getUTCFullYear() + " " + (t.getUTCMonth()+1)+" "+t.getUTCDate()+" "+t.getUTCHours()+" "+t.getUTCMinutes()+" 0");
		});
	});

	$(".uv-annualmotion-slider").each(function(index) {

		$(this).change( function() {
			annualMotion = $(this).val() / 365;
			var t = new Date(t0_year, t0_month, t0_date, t0_hour, t0_min, 0, 0);
			t.setTime(t.getTime() + annualMotion*365*24*60*60*1000 + dailyMotion*24*60*60*1000 + tz_off*60*1000);

			$(".uv-annualmotion-slider").val($(this).val());

			$(".uv-annualmotion-slider-printout").text("Date: "+t.getUTCFullYear()+ "/" + (t.getUTCMonth()+1) + "/" + t.getUTCDate());
			sendMessage("system.setsimulatedtime 2012 "+ (t.getUTCMonth()+1) + " " + t.getUTCDate()+" "+t.getUTCHours()+" "+t.getUTCMinutes()+" 0");
		});
	});

	$(".uv-siderealmotion-slider").each(function(index) {

		$(this).change( function() {
			annualMotion = ($(this).val() / 365)*0.997269566666;
			var t = new Date(t0_year, t0_month, t0_date, t0_hour, t0_min, 0, 0);
			t.setTime(t.getTime() + annualMotion*365*24*60*60*1000 + dailyMotion*24*60*60*1000 + tz_off*60*1000);

			$(".uv-siderealmotion-slider").val($(this).val());

			$(".uv-siderealmotion-slider-printout").text("Date: "+t.getUTCFullYear()+ "/" + (t.getUTCMonth()+1) + "/" + t.getUTCDate());
			sendMessage("system.setsimulatedtime 2012 "+ (t.getUTCMonth()+1) + " " + t.getUTCDate()+" "+t.getUTCHours()+" "+t.getUTCMinutes()+" 0");
		});
	});

	/*
		Float message sliders:

		class="uv-float-message-slider" data-message="%v" data-mouseup-message=""

		Could be used for camera movement or other things not (yet) in properties.

		Will replace every occurance of %v with the value of the slider.

		Creates a slider out of the div
	*/

	$(".uv-float-message-slider").each(function(index) {
		var message = $(this).attr('data-message');
		var mouseupmessage = $(this).attr('data-mouseup-message');

		var releaseval = $(this).attr('data-mouseup-value');
		if (!releaseval || releaseval == "")
			releaseval = 0.0;

		$(this).change(function() {
			var val = $(this).val();
			var smsg = message.replace("%v", val);
			sendMessage(smsg);
		});

		$(this).mouseup(function() {
			sendMessage(mouseupmessage);
			$(this).val(releaseval);
		});

		$(this).bind("touchend", function(e) {
			sendMessage(mouseupmessage);
			$(this).val(releaseval);
		});
	});

});

